#include common_scripts\utility;

bcs_location_trigs_init()
{
	// create array if it hasn't been done by the dds system from _dds::dds_init()
	// having it created in both places is temporarily.  I plan to remove the mw3 battlechatter later
	if (IsDefined(level.dds) && IsDefined(anim.bcs_locations))
		return;
		
	ASSERT( !IsDefined( level.bcs_location_mappings ) );
	level.bcs_location_mappings = [];
	
	bcs_location_trigger_mapping();	
	bcs_trigs_assign_aliases();
	
	// now that the trigger ents have their aliases set on them, clear out our big array
	//  so we can save on script variables
	level.bcs_location_mappings = undefined;
	
	anim.locationLastCalloutTimes = [];
}

bcs_trigs_assign_aliases()
{
	ASSERT( !IsDefined( anim.bcs_locations ) );
	anim.bcs_locations = [];
	
	ents = GetEntArray();
	trigs = [];
	foreach( trig in ents )
	{
		if( IsDefined( trig.classname ) && IsSubStr( trig.classname, "trigger_multiple_bcs" ) )
		{
			trigs[ trigs.size ] = trig;
		}
	}
	
	foreach( trig in trigs )
	{
		if ( !IsDefined( level.bcs_location_mappings[ trig.classname ] ) )
		{
			/#
			// iPrintln( "^2" + "WARNING: Couldn't find bcs location mapping for battlechatter trigger with classname " + trig.classname );
			// do nothing since too many prints kills the command buffer
			#/
		}
		else
		{
			aliases = ParseLocationAliases( level.bcs_location_mappings[ trig.classname ] );
			if( aliases.size > 1 )
			{
				aliases = array_randomize( aliases );
			}
			
			trig.locationAliases = aliases;
		}
	}
	
	anim.bcs_locations = trigs;
}

// parses locationStr using a space as a token and returns an array of the data in that field
ParseLocationAliases( locationStr )
{
	locationAliases = StrTok( locationStr, " " );
	return locationAliases;
}

add_bcs_location_mapping( classname, alias )
{
	// see if we have to add to an existing entry
	if( IsDefined( level.bcs_location_mappings[ classname ] ) )
	{
		existing = level.bcs_location_mappings[ classname ];
		existingArr = ParseLocationAliases( existing );
		aliases = ParseLocationAliases( alias );
		
		foreach( a in aliases )
		{
			foreach( e in existingArr )
			{
				if( a == e )
				{
					return;
				}
			}
		}
		
		existing += " " + alias;
		level.bcs_location_mappings[ classname ] = existing;
		
		return;
	}
	
	// otherwise make a new entry
	level.bcs_location_mappings[ classname ] = alias;
}


// here's where we set up each kind of trigger and map them to their (partial) soundaliases
bcs_location_trigger_mapping()
{
	if ( isSP() )
	{
		generic_locations();

		fusion_locations();
		sanfran_locations();
		sanfran_b_locations();
		greece_locations();
		seoul_locations();
		detroit_locations();
		betrayal_locations();
		lab_locations();
		recovery_locations();
		lagos_locations();
	
		merida();
	
		old_locations();
	}
	else
	{
		laser2_mp();
		lab2_mp();
		mp_refraction();
		mp_prison();
		mp_dam(); 
		mp_detroit();
		mp_greenband();
		mp_instinct();
		mp_levity();
		mp_recovery();
		mp_solar();
		mp_terrace();
		mp_torqued();
		mp_venus();
		mp_comeback();
		old_locations_mp();
	}
}

//---------------------------------------------------------
// GENERICS
//---------------------------------------------------------
generic_locations()
{
/*QUAKED trigger_multiple_bcs_generic_doorway_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="doorway_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_doorway_generic", "doorway_generic" );

/*QUAKED trigger_multiple_bcs_generic_window_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="window_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_window_generic", "window_generic" );

/*QUAKED trigger_multiple_bcs_generic_1stfloor_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="1stfloor_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_1stfloor_generic", "1stfloor_generic" );

/*QUAKED trigger_multiple_bcs_generic_1stfloor_doorway (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="1stfloor_doorway"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_1stfloor_doorway", "1stfloor_doorway" );

/*QUAKED trigger_multiple_bcs_generic_1stfloor_window (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="1stfloor_window"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_1stfloor_window", "1stfloor_window" );

/*QUAKED trigger_multiple_bcs_generic_2ndfloor_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="2ndfloor_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_2ndfloor_generic", "2ndfloor_generic" );

/*QUAKED trigger_multiple_bcs_generic_2ndfloor_window (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="2ndfloor_window"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_2ndfloor_window", "2ndfloor_window" );

/*QUAKED trigger_multiple_bcs_generic_rooftop (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rooftop"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_rooftop", "rooftop" );

/*QUAKED trigger_multiple_bcs_generic_2ndfloor_balcony (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="2ndfloor_balcony"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_generic_2ndfloor_balcony", "2ndfloor_balcony" );
}

//------------------------------//
//-------XSLICE BCS Trigs-------//
//------------------------------//

fusion_locations()
{
/*QUAKED trigger_multiple_bcs_fus_truck (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_truck"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_truck", "fus_truck" );
	
/*QUAKED trigger_multiple_bcs_fus_tower (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_tower"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_tower", "fus_tower" );
	
/*QUAKED trigger_multiple_bcs_fus_generator (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_generator"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_generator", "fus_generator" );
	
/*QUAKED trigger_multiple_bcs_fus_mt (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_mt"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_mt", "fus_mt" );
	
/*QUAKED trigger_multiple_bcs_fus_titan (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_titan"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_titan", "fus_titan" );
	
/*QUAKED trigger_multiple_bcs_fus_hill (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_hill"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_hill", "fus_hill" );
	
/*QUAKED trigger_multiple_bcs_fus_garage (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_garage"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_garage", "fus_garage" );
	
/*QUAKED trigger_multiple_bcs_fus_crane (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_crane"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_crane", "fus_crane" );
	
/*QUAKED trigger_multiple_bcs_fus_forklift (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_forklift"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_forklift", "fus_forklift" );
	
/*QUAKED trigger_multiple_bcs_fus_pillar (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_pillar"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_pillar", "fus_pillar" );
	
/*QUAKED trigger_multiple_bcs_fus_pipes (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_pipes"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_pipes", "fus_pipes" );
	
/*QUAKED trigger_multiple_bcs_fus_balcony (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_balcony"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_balcony", "fus_balcony" );
	
/*QUAKED trigger_multiple_bcs_fus_catwalk (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_catwalk"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_catwalk", "fus_catwalk" );
	
/*QUAKED trigger_multiple_bcs_fus_fueltrucks (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_fueltrucks"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_fueltrucks", "fus_fueltrucks" );
	
/*QUAKED trigger_multiple_bcs_fus_walkway (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_walkway"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_walkway", "fus_walkway" );
	
/*QUAKED trigger_multiple_bcs_fus_stairs (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fus_stairs"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_fus_stairs", "fus_stairs" );	
}

sanfran_locations()
{
	
/*QUAKED trigger_multiple_bcs_sfa_bus (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_bus"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_bus", "sfa_bus" );
	
/*QUAKED trigger_multiple_bcs_sfa_cardoor (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_cardoor"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_cardoor", "sfa_cardoor" );
	
/*QUAKED trigger_multiple_bcs_sfa_cargovan (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_cargovan"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_cargovan", "sfa_cargovan" );
	
/*QUAKED trigger_multiple_bcs_sfa_policecar (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_policecar"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_policecar", "sfa_policecar" );
	
/*QUAKED trigger_multiple_bcs_sfa_roadsign (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_roadsign"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_roadsign", "sfa_roadsign" );
	
/*QUAKED trigger_multiple_bcs_sfa_sidewalk (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_sidewalk"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_sidewalk", "sfa_sidewalk" );
	
/*QUAKED trigger_multiple_bcs_sfa_sportscar (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_sportscar"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_sportscar", "sfa_sportscar" );
	
/*QUAKED trigger_multiple_bcs_sfa_topbus (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_topbus"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_topbus", "sfa_topbus" );
	
/*QUAKED trigger_multiple_bcs_sfa_tower (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_tower"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_tower", "sfa_tower" );
	
/*QUAKED trigger_multiple_bcs_sfa_trailer (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfa_trailer"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfa_trailer", "sfa_trailer" );
	
	/*QUAKED trigger_multiple_bcs_sfa_truck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="sfa_truck"
*/
	add_bcs_location_mapping ("trigger_multiple_bcs_sfa_truck", "sfa_truck");
		
}

sanfran_b_locations()
{
	
/*QUAKED trigger_multiple_bcs_sfb_above (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="above"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_above", "above" );
	
/*QUAKED trigger_multiple_bcs_sfb_containers (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfb_containers"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_containers", "sfb_containers" );
	
/*QUAKED trigger_multiple_bcs_sfb_doors (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="doors"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_doors", "doors" );
	
/*QUAKED trigger_multiple_bcs_sfb_helipad (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="helipad"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_helipad", "helipad" );
	
/*QUAKED trigger_multiple_bcs_sfb_missileturret (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="missileturret"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_missileturret", "missileturret" );
	
/*QUAKED trigger_multiple_bcs_sfb_table (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="table"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_table", "table" );
	
/*QUAKED trigger_multiple_bcs_sfb_truck (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfb_truck"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_truck", "sfb_truck" );
	
/*QUAKED trigger_multiple_bcs_sfb_vtol (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="vtol"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_vtol", "vtol" );
	
/*QUAKED trigger_multiple_bcs_sfb_wreckage (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="wreckage"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_wreckage", "wreckage" );
	
/*QUAKED trigger_multiple_bcs_sfb_cargocrane (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cargocrane"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_cargocrane", "cargocrane" );
	
/*QUAKED trigger_multiple_bcs_sfb_catwalk (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfb_catwalk"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_catwalk", "sfb_catwalk" );
	
/*QUAKED trigger_multiple_bcs_sfb_console (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfb_console"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_console", "sfb_console" );
	
/*QUAKED trigger_multiple_bcs_sfb_jammer (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfb_jammer"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_jammer", "sfb_jammer" );
	
/*QUAKED trigger_multiple_bcs_sfb_launchpad (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfb_launchpad"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_launchpad", "sfb_launchpad" );
	
/*QUAKED trigger_multiple_bcs_sfb_jet (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sfb_jet"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_sfb_jet", "sfb_jet" );
		
}

greece_locations()
{
/*QUAKED trigger_multiple_bcs_grk_balcony (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="grk_balcony"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_grk_balcony", "grk_balcony" );

/*QUAKED trigger_multiple_bcs_grk_1stfloor_window (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="grk_1stfloor_window"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_grk_1stfloor_window", "grk_1stfloor_window" );

/*QUAKED trigger_multiple_bcs_grk_2ndfloor_window (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="grk_2ndfloor_window"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_grk_2ndfloor_window", "grk_2ndfloor_window" );

/*QUAKED trigger_multiple_bcs_grk_3rdfloor_window (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="grk_3rdfloor_window"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_grk_3rdfloor_window", "grk_3rdfloor_window" );
	
/*QUAKED trigger_multiple_bcs_grk_cafe (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="grk_cafe"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_grk_cafe", "grk_cafe" );

/*QUAKED trigger_multiple_bcs_grk_rooftop (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="grk_rooftop"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_grk_rooftop", "grk_rooftop" );
	
	/*QUAKED trigger_multiple_bcs_grk_doorway (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="grk_doorway"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_grk_doorway", "grk_doorway" );
	
/*QUAKED trigger_multiple_bcs_gre_vehic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="gre_vehic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_gre_vehic", "gre_vehic" );
	
/*QUAKED trigger_multiple_bcs_gre_statue (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="gre_statue"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_gre_statue", "gre_statue" );
}

seoul_locations()
{
	/*QUAKED trigger_multiple_bcs_seo_balcony (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_balcony"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_balcony", "seo_balcony");
	
	/*QUAKED trigger_multiple_bcs_seo_bus (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_bus"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_bus", "seo_bus");
	
	/*QUAKED trigger_multiple_bcs_seo_stairs (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_stairs"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_stairs", "seo_stairs");
	
	/*QUAKED trigger_multiple_bcs_seo_2ndwindow (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_2ndwindow"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_2ndwindow", "seo_2ndwindow");
	
	/*QUAKED trigger_multiple_bcs_seo_3rdbalcony (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_3rdbalcony"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_3rdbalcony", "seo_3rdbalcony");
	
	/*QUAKED trigger_multiple_bcs_seo_sculpture (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_sculpture"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_sculpture", "seo_sculpture");
	
	/*QUAKED trigger_multiple_bcs_seo_window (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_window"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_window", "seo_window");
	
	/*QUAKED trigger_multiple_bcs_seo_window (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_window"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_window", "seo_window");
	
	/*QUAKED trigger_multiple_bcs_seo_cafe (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_cafe"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_cafe",	"seo_cafe");
	
	/*QUAKED trigger_multiple_bcs_seo_counter (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="seo_counter"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_seo_counter", "seo_counter");

}

detroit_locations()
{
	/*QUAKED trigger_multiple_bcs_det_bar (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="det_bar"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_det_bar", "det_bar");

	/*QUAKED trigger_multiple_bcs_det_balcony (0 0.25 0.5) ? 
defaulttexture="bcs"
soundalias="det_balcony"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_det_balcony", "det_balcony");	

	/*QUAKED trigger_multiple_bcs_det_walkway (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="det_walkway"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_det_walkway", "det_walkway");
	
	/*QUAKED trigger_multiple_bcs_det_ambulance (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="det_ambulance"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_det_ambulance", "det_ambulance");
}

betrayal_locations()
{
	/*QUAKED trigger_multiple_bcs_bet_plaza (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_plaza"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_plaza", "bet_plaza");
	
	/*QUAKED trigger_multiple_bcs_bet_way (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_way"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_way", "bet_way");
	
	/*QUAKED trigger_multiple_bcs_bet_1stfloor (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_1stfloor"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_1stfloor", "bet_1stfloor");
	
	/*QUAKED trigger_multiple_bcs_bet_2ndfloor (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_2ndfloor"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_2ndfloor", "bet_2ndfloor");
	
	/*QUAKED trigger_multiple_bcs_bet_3rdfloor (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_3rdfloor"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_3rdfloor", "bet_3rdfloor");
	
	/*QUAKED trigger_multiple_bcs_bet_above (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_above"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_above", "bet_above");
	
	/*QUAKED trigger_multiple_bcs_bet_balcony (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_balcony"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_balcony", "bet_balcony");
	
	/*QUAKED trigger_multiple_bcs_bet_barge (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_barge"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_barge", "bet_barge");
	
	/*QUAKED trigger_multiple_bcs_bet_checkpt (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_checkpt"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_checkpt", "bet_checkpt");
	
	/*QUAKED trigger_multiple_bcs_bet_doorway (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_doorway"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_doorway", "bet_doorway");
	
	/*QUAKED trigger_multiple_bcs_bet_open (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_open"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_open", "bet_open");
	
	/*QUAKED trigger_multiple_bcs_bet_patio (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_patio"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_patio", "bet_patio");
	
	/*QUAKED trigger_multiple_bcs_bet_rooftop (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_rooftop"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_rooftop", "bet_rooftop");
	
	/*QUAKED trigger_multiple_bcs_bet_street (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_street"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_street", "bet_street");
	
	/*QUAKED trigger_multiple_bcs_bet_drones (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_drones"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_drones", "bet_drones");
	
	/*QUAKED trigger_multiple_bcs_bet_fountain (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_fountain"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_fountain", "bet_fountain");
	
	/*QUAKED trigger_multiple_bcs_bet_skybridge (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_skybridge"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_skybridge", "bet_skybridge");
	
	/*QUAKED trigger_multiple_bcs_bet_below (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_below"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_below", "bet_below");
	
	/*QUAKED trigger_multiple_bcs_bet_bldng (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_bldng"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_bldng", "bet_bldng");
	
	/*QUAKED trigger_multiple_bcs_bet_bridge (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_bridge"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_bridge", "bet_bridge");
	
	/*QUAKED trigger_multiple_bcs_bet_deplycover (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_deplycover"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_deplycover", "bet_deplycover");
	
	/*QUAKED trigger_multiple_bcs_bet_pallets (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_pallets"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_pallets", "bet_pallets");
	
	/*QUAKED trigger_multiple_bcs_bet_catwalk (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_catwalk"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_catwalk", "bet_catwalk");
	
	/*QUAKED trigger_multiple_bcs_bet_table (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_table"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_table", "bet_table");
	
	/*QUAKED trigger_multiple_bcs_bet_crates (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_crates"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_crates", "bet_crates");
	
	/*QUAKED trigger_multiple_bcs_bet_dock (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_dock"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_dock", "bet_dock");
	
	/*QUAKED trigger_multiple_bcs_bet_leftshore (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_leftshore"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_leftshore", "bet_leftshore");
	
	/*QUAKED trigger_multiple_bcs_bet_rightshore (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bet_rightshore"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_bet_rightshore", "bet_rightshore");
}

lab_locations()
{
	/*QUAKED trigger_multiple_bcs_lab_canisters (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lab_canisters"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lab_canisters", "lab_canisters");
	
	/*QUAKED trigger_multiple_bcs_lab_camera (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lab_camera"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lab_camera", "lab_camera");
	
	/*QUAKED trigger_multiple_bcs_lab_van (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lab_van"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lab_van", "lab_van");
	
	/*QUAKED trigger_multiple_bcs_lab_lwrcatwalk (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lab_lwrcatwalk"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lab_lwrcatwalk", "lab_lwrcatwalk");
	
	/*QUAKED trigger_multiple_bcs_lab_uprcatwalk (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lab_uprcatwalk"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lab_uprcatwalk", "lab_uprcatwalk");
	
	/*QUAKED trigger_multiple_bcs_lab_forklift (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lab_forklift"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lab_forklift", "lab_forklift");
	
	/*QUAKED trigger_multiple_bcs_lab_rooftop (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lab_rooftop"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lab_rooftop", "lab_rooftop");
}

recovery_locations()
{
	/*QUAKED trigger_multiple_bcs_rec_firepit (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rec_firepit"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_rec_firepit", "rec_firepit");
	
	/*QUAKED trigger_multiple_bcs_rec_hill (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rec_hill"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_rec_hill", "rec_hill");
	
	/*QUAKED trigger_multiple_bcs_rec_pool (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rec_pool"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_rec_pool", "rec_pool");
	
	/*QUAKED trigger_multiple_bcs_rec_road (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rec_road"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_rec_road", "rec_road");
	
	/*QUAKED trigger_multiple_bcs_rec_patio (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rec_patio"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_rec_patio", "rec_patio");
	
	/*QUAKED trigger_multiple_bcs_rec_table (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rec_table"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_rec_table", "rec_table");
	
	/*QUAKED trigger_multiple_bcs_rec_couch (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rec_couch"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_rec_couch", "rec_couch");
}

lagos_locations()
{
	/*QUAKED trigger_multiple_bcs_lag_street (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lag_street"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lag_street", "lag_street");
	
	/*QUAKED trigger_multiple_bcs_lag_dogs (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lag_dogs"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lag_dogs", "lag_dogs");
	
	/*QUAKED trigger_multiple_bcs_lag_median (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lag_median"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lag_median", "lag_median");
	
	/*QUAKED trigger_multiple_bcs_lag_rpgbus (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lag_rpgbus"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lag_rpgbus", "lag_rpgbus");
	
	/*QUAKED trigger_multiple_bcs_lag_overpass (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lag_overpass"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lag_overpass", "lag_overpass");
	
	/*QUAKED trigger_multiple_bcs_lag_suv (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lag_suv"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lag_suv", "lag_suv");
	
	/*QUAKED trigger_multiple_bcs_lag_suvapproach (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lag_suvapproach"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lag_suvapproach", "lag_suvapproach");
	
	/*QUAKED trigger_multiple_bcs_lag_topvan (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lag_topvan"
*/
	add_bcs_location_mapping ( "trigger_multiple_bcs_lag_topvan", "lag_topvan");
}
	
merida()
{
/*QUAKED trigger_multiple_bcs_mp_merida_radiotower (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="radiotower"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_radiotower", "radiotower" );

/*QUAKED trigger_multiple_bcs_mp_merida_embassy_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="embassy_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_embassy_generic", "embassy_generic" );

/*QUAKED trigger_multiple_bcs_mp_merida_aaguns (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="aaguns"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_aaguns", "aaguns" );

/*QUAKED trigger_multiple_bcs_mp_merida_tunnel (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="tunnel"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_tunnel", "tunnel" );

/*QUAKED trigger_multiple_bcs_mp_merida_cannons_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cannons_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_cannons_generic", "cannons_generic" );

/*QUAKED trigger_multiple_bcs_mp_merida_pool (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="pool"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_pool", "pool" );

/*QUAKED trigger_multiple_bcs_mp_merida_embassy_north (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="embassy_north"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_embassy_north", "embassy_north" );

/*QUAKED trigger_multiple_bcs_mp_merida_embassy_south (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="embassy_south"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_embassy_south", "embassy_south" );

/*QUAKED trigger_multiple_bcs_mp_merida_embassy_east (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="embassy_east"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_embassy_east", "embassy_east" );

/*QUAKED trigger_multiple_bcs_mp_merida_embassy_west (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="embassy_west"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_embassy_west", "embassy_west" );

/*QUAKED trigger_multiple_bcs_mp_merida_cannons_embassy (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cannons_embassy"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_cannons_embassy", "cannons_embassy" );

/*QUAKED trigger_multiple_bcs_mp_merida_cannons_radiotower (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cannons_radiotower"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_merida_cannons_radiotower", "cannons_radiotower" );
}

//---------------------------------------------------------
// - BLACKSMITH MULTIPLAYER -
//---------------------------------------------------------

laser2_mp()
{
	
/*QUAKED trigger_multiple_bcs_mp_lsr_radardish (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lsr_radardish"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_radardish",	"lsr_radardish" );

/*QUAKED trigger_multiple_bcs_mp_lsr_laserairdefensegun (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lsr_laserairdefensegun"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_laserairdefensegun", "lsr_laserairdefensegun" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_razorback (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lsr_razorback"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_razorback", "lsr_razorback" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_underhelipad (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_underhelipad"
*/
	
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_underhelipad", "lsr_underhelipad" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_bunkerlookout (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_bunkerlookout"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_bunkerlookout", "lsr_bunkerlookout" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_inradartower (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_inradartower"
*/
	
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_inradartower", "lsr_inradartower" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_byradartower (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_byradartower"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_byradartower", "lsr_byradartower" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_shippingcontainer (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_shippingcontainer"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_shippingcontainer", "lsr_shippingcontainer" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_onhelipad (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_onhelipad"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_onhelipad", "lsr_onhelipad" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_onbeach (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_onbeach"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_onbeach", "lsr_onbeach" );
	
/*QUAKED trigger_multiple_bcs_mp_lsr_bybridge (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_bybridge"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_onbeach", "lsr_bybridge");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_bylasergenerator (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_bylasergenerator"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_bylasergenerator", "lsr_bylasergenerator");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_underlaser (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_underlaser"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_underlaser", "lsr_underlaser");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_byhelipad (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_byhelipad"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_byhelipad", "lsr_byhelipad");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_electricalroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_electricalroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_electricalroom", "lsr_electricalroom");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_piperoom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_piperoom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_piperoom", "lsr_piperoom");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_bybuoy (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_bybuoy"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_bybuoy", "lsr_bybuoy");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_bygenerator (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_bygenerator"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_bygenerator", "lsr_bygenerator");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_offswitch (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_offswitch"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_offswitch", "lsr_offswitch");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_missilerack (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_missilerack"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_missilerack", "lsr_missilerack");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_underchains (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_underchains"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_underchains", "lsr_underchains");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_beachbunkerrooftop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_beachbunkerrooftop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_beachbunkerrooftop", "lsr_beachbunkerrooftop");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_bunkerrooftop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_bunkerrooftop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_bunkerrooftop", "lsr_bunkerrooftop");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_nettedcargo (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_nettedcargo"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_nettedcargo", "lsr_nettedcargo");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_cagedcargo (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_cagedcargo"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_cagedcargo", "lsr_cagedcargo");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_overturnedcar (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_overturnedcar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_overturnedcar", "lsr_overturnedcar");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_bridgebase (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_bridgebase"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_bridgebase", "lsr_bridgebase");
	
/*QUAKED trigger_multiple_bcs_mp_lsr_cornerbunker (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lsr_cornerbunker"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lsr_cornerbunker", "lsr_cornerbunker");
}

lab2_mp()
{
	
/*QUAKED trigger_multiple_bcs_mp_lab_parkinglot (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_parkinglot"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_parkinglot", "lab_parkinglot");
	
/*QUAKED trigger_multiple_bcs_mp_lab_serverroom	(0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_serverroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_serverroom",	"lab_serverroom");
	
/*QUAKED trigger_multiple_bcs_mp_lab_holoroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_holoroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_holoroom", "lab_holoroom");
	
/*QUAKED trigger_multiple_bcs_mp_lab_dryingroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_dryingroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_dryingroom", "lab_dryingroom");
	
/*QUAKED trigger_multiple_bcs_mp_lab_chemvats	(0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_chemvats"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_chemvats", "lab_chemvats");
	
/*QUAKED trigger_multiple_bcs_mp_lab_generator (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_generator"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_generator", "lab_generator");
	
/*QUAKED trigger_multiple_bcs_mp_lab_lobby (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_generator"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_lobby", "lab_lobby");

/*QUAKED trigger_multiple_bcs_mp_lab_behindcrates (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_behindcrates"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_behindcrates", "lab_behindcrates");

/*QUAKED trigger_multiple_bcs_mp_lab_scienceroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_scienceroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_scienceroom", "lab_scienceroom");
	
/*QUAKED trigger_multiple_bcs_mp_lab_storageroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_storageroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_storageroom", "lab_storageroom");
	
/*QUAKED trigger_multiple_bcs_mp_lab_trench (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_trench"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_trench", "lab_trench");

/*QUAKED trigger_multiple_bcs_mp_lab_behindvehicle (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lab_behindvehicle"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lab_behindvehicle", "lab_behindvehicle");
} 


mp_refraction()
{
/*QUAKED trigger_multiple_bcs_mp_ref_oncrane (0 0.25 0.5)?
 defaulttexture="bcs"
 soundalias="ref_oncrane"
 */
 
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_oncrane", "ref_oncrane");
	
/*QUAKED trigger_multiple_bcs_mp_ref_byloadingdocks (0 0.25 0.5)?
defaulttexture="bcs" 
soundalias="ref_byloadingdocks"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_byloadingdocks", "ref_byloadingdocks");
	
/*QUAKED trigger_multiple_bcs_mp_ref_nearelevatorentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_nearelevatorentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_nearelevatorentrance", "ref_nearelevatorentrance");
	
/*QUAKED trigger_multiple_bcs_mp_ref_inelevator (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_inelevator"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_inelevator", "ref_inelevator");
	
/*QUAKED trigger_multiple_bcs_mp_ref_oncatwalks (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_oncatwalks"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_oncatwalks",	"ref_oncatwalks");
	
/*QUAKED trigger_multiple_bcs_mp_ref_seccheckpoint (0 0.25 0.5)?
defaulttextture="bcs"
soundalias="ref_seccheckpoint"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_seccheckpoint", "ref_seccheckpoint");
	
/*QUAKED trigger_multiple_bcs_mp_ref_helopadentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_helopadentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_helopadentrance", "ref_helopadentracne");
	
/*QUAKED trigger_multiple_bcs_mp_ref_checkinstation (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_checkinstation"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_checkinstation", "ref_checkinstation");

/*QUAKED trigger_multiple_bcs_mp_ref_maintenaceentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_maintenceentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_maintenceentrance", "ref_maintenceentrance");
	
/*QUAKED trigger_multiple_bcs_mp_ref_insidelounge (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_insidelounge"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_insidelounge", "ref_insidelounge");
	
/*QUAKED trigger_multiple_bcs_mp_ref_bylockmechanism (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_bylockmechanism"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_bylockmechanism", "ref_bylockmechanism");

/*QUAKED trigger_multiple_bcs_mp_ref_inelevatorshaft (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_inelevatorshaft"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_inelevatorshaft", "ref_inelevatorshaft");
	
/*QUAKED trigger_multiple_bcs_mp_ref_underhelipad (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_underhelipad"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_underhelipad", "ref_underhelipad");
	
/*QUAKED trigger_multiple_bcs_mp_ref_bylgcontainers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_bylgcontainers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_bylgcontainers", "ref_bylgcontainers");
	
/*QUAKED trigger_multiple_bcs_mp_ref_bylockers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_bylockers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_bylockers", "ref_bylockers");
	
/*QUAKED trigger_multiple_bcs_mp_ref_abovelockers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_abovelockers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_abovelockers", "ref_abovelockers");
	
/*QUAKED trigger_multiple_bcs_mp_ref_byadtowers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ref_byadtowers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ref_byadtowers", "ref_byadtowers");
}

mp_prison()
{	
/*QUAKED trigger_multiple_bcs_mp_psn_insidecellblock (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_insidecellblock"
*/

	add_bcs_location_mapping(" trigger_multiple_bcs_mp_psn_insidecellblock", "psn_insidecellblock");
	
/*QUAKED trigger_multiple_bcs_mp_psn_psnfrontentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_psnfrontentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_psnfrontentrance", "psn_psnfrontentrance");
	
/*QUAKED trigger_multiple_bcs_mp_psn_insidemaintenancebldg (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_insidemaintenancebldg"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_insidemaintenancebldg", "psn_insidemaintenancebldg");
	
/*QUAKED trigger_multiple_bcs_mp_psn_onmaintenancebldgroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_onmaintenancebldgroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_onmaintenancebldgroof", "psn_onmaintenancebldgroof");
	
/*QUAKED trigger_multiple_bcs_mp_psn_prisonyard (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_prisonyard"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_prisonyard", "psn_prisonyard");
	
/*QUAKED trigger_multiple_bcs_mp_psn_bballcourt (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_bballcourt"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_bballcourt", "psn_bballcourt");
	
/*QUAKED trigger_multiple_bcs_mp_psn_psnbus (0 0.25 0.5)?
defaulttexture="bcs" 
soundalias="psn_psnbus"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_psnbus", "psn_psnbus");
	
/*QUAKED trigger_multiple_bcs_mp_psn_sectower (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_sectower"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_sectower", "psn_sectower");
	
/*QUAKED trigger_multiple_bcs_mp_psn_insidegarage (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_insidegarage"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_insidegarage", "psn_insidegarage");
	
/*QUAKED trigger_multiple_bcs_mp_psn_topofgarage (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_topofgarage"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_topofgarage", "psn_topofgarage");
	
/*QUAKED trigger_multiple_bcs_mp_psn_destroyedwall (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_destroyedwall"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_destroyedwall", "psn_destroyedwall");
	
	
/*QUAKED trigger_multiple_bcs_mp_psn_cellblockcatwalk (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_cellblockcatwalk"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_cellblockcatwalk", "psn_cellblockcatwalk");

/*QUAKED trigger_multiple_bcs_mp_psn_lobbyentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_lobbyentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_lobbyentrance", "psn_lobbyentrance");

/*QUAKED trigger_multiple_bcs_mp_psn_electricalbox (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_electricalbox"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_electricalbox", "psn_electricalbox");
	
/*QUAKED trigger_multiple_bcs_mp_psn_policecar (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_policecar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_policecar", "psn_policecar");
	
/*QUAKED trigger_multiple_bcs_mp_psn_behinddumpster (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_behinddumpster"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_behinddumpster", "psn_behinddumpster");

/*QUAKED trigger_multiple_bcs_mp_psn_catwalk (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_catwalk"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_catwalk", "psn_catwalk");
	
/*QUAKED trigger_multiple_bcs_mp_psn_laundrybin (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_laundrybin"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_laundrybin", "psn_laundrybin");
	
/*QUAKED trigger_multiple_bcs_mp_psn_psnrooftop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_psnrooftop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_psnrooftop", "psn_psnrooftop");
	
/*QUAKED trigger_multiple_bcs_mp_psn_behindac (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_behindac"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_behindac", "psn_behindac");
	
/*QUAKED trigger_multiple_bcs_mp_psn_insidelockerbldg (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="psn_insidelockerbldg"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_psn_insidelockerbldg", "psn_insidelockerbldg");
}

mp_dam()
{
/*QUAKED trigger_multiple_bcs_mp_dam_frontgate (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_frontgate"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_frontgate", "dam_frontgate");
	
/*QUAKED trigger_multiple_bcs_mp_dam_semitruck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_semitruck"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_semitruck", "dam_semitruck");
	
/*QUAKED trigger_multiple_bcs_mp_dam_constructionsite (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_constructionsite"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_contructionsite", "dam_contructionsite");
	
/*QUAKED trigger_multiple_bcs_mp_dam_insideoffice (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_insideoffice"
*/
	
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_insideoffice", "dam_insideoffice");
	
/*QUAKED trigger_multiple_bcs_mp_dam_oncatwalks (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_oncatwalks"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_oncatwalks", "dam_oncatwalks");

/*QUAKED trigger_multiple_bcs_mp_dam_centerstreet (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_centerstreet"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_centerstreet", "dam_centerstreet");
	
/*QUAKED trigger_multiple_bcs_mp_dam_onminigun (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_onminigun"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_onminigun", "dam_onminigun");
	
/*QUAKED trigger_multiple_bcs_mp_dam_cranebase (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_cranebase"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_cranebase", "dam_cranebase");

/*QUAKED trigger_multiple_bcs_mp_dam_infactory (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_infactory"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_infactory", "dam_infactory");
	
/*QUAKED trigger_multiple_bcs_mp_dam_inturbinebldg (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_inturbinebldg"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_inturbinebldg", "dam_inturbinebldg");
	
/*QUAKED trigger_multiple_bcs_mp_dam_movingpipe (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_movingpiep"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_movingpipe", "dam_movingpipe");
	
/*QUAKED trigger_multiple_bcs_mp_dam_electricalgrid (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_electricalgrid"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_electricalgrid", "dam_electricalgrid");
	
/*QUAKED trigger_multiple_bcs_mp_dam_insidelounge (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_insidelounge"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_dam_insidelounge", "dam_insidelounge");
	
/*QUAKED trigger_multiple_bcs_mp_dam_inpipes (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_inpipes"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_inpipes", "dam_inpipes");
	
/*QUAKED trigger_multiple_bcs_mp_dam_underground (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="dam_underground"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_underground", "dam_underground");
	
/*QUAKED trigger_multiple_bcs_mp_dam_factoryrooftop (0 0.25 0.5)?
defaulttexture="bcs"	
soundalias="dam_factoryrooftop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dam_factoryrooftop", "dam_factoryrooftop");

}

mp_detroit()
{
/*QUAKED trigger_multiple_bcs_mp_det_inschool (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_inschool"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_inschool", "det_inschool");
	
/*QUAKED trigger_multiple_bcs_mp_det_bylockers (0 0.25 0.5)?
defaulttexture="bcs"	
soundalias="det_bylockers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_bylockers", "det_bylockers");
	
/*QUAKED trigger_multiple_bcs_mp_det_hospitalentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_hospitalentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_hospitalentrance", "det_hospitalentrance");
	
/*QUAKED trigger_multiple_bcs_mp_det_hospitallobby (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_hospitallobby"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_hospitallobby", "det_hospitallobby");
	
/*QUAKED trigger_multiple_bcs_mp_det_inparkinggarage (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_inparkinggarage"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_inparkinggarage", "det_inparkinggarage");
	
/*QUAKED trigger_multiple_bcs_mp_det_garageentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_garageentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_garageentrance", "det_garageentrance");
	
/*QUAKED trigger_multiple_bcs_mp_det_piperoom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_piperoom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_piperoom", "det_piperoom");
	
/*QUAKED trigger_multiple_bcs_mp_det_onramp (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_onramp"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_onramp", "det_onramp");
	
/*QUAKED trigger_multiple_bcs_mp_det_inalley (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_inalley"
*/ 

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_inalley", "det_inalley");
	
/*QUAKED trigger_multiple_bcs_mp_det_bytrailers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_bytrailers" 
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_bytrailers", "det_bytrailers");
	
/*QUAKED trigger_multiple_bcs_mp_det_ontrailers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_ontrailers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_ontrailers", "det_ontrailers");
	
/*QUAKED trigger_multiple_bcs_mp_det_inthepod (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_inthepod"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_inthepod", "det_inthepod");
	
/*QUAKED trigger_multiple_bcs_mp_det_throughpark (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_throughpark"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_throughpark", "det_throughpark");
	
/*QUAKED trigger_multiple_bcs_mp_det_byplayground (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_byplayground"
*/

add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_byplayground", "det_byplayground");
							
/*QUAKED trigger_multiple_bcs_mp_det_garageoverlook (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_garageoverlook"
*/

	add_bcs_location_mapping( "triger_multiple_bcs_mp_det_garageoverlook", "det_garageoverlook");
	
/*QUAKED trigger_multiple_bcs_mp_det_backalley (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_backalley"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_backalley", "det_backalley");
	
/*QUAKED trigger_multiple_bcs_mp_det_parkoffice (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_parkoffice"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_parkoffice", "det_parkoffice");
	
/*QUAKED trigger_multiple_bcs_mp_det_hospitaloffice (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_hospitaloffice"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_hospitaloffice", "det_hospitaloffice");
	
/*QUAKED trigger_multiple_bcs_mp_det_throughstreet (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_throughstreet"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_throughstreet", "det_throughstreet");
	
/*QUAKED trigger_multiple_bcs_mp_det_upperstreet (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_upperstreet"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_upperstreet", "det_upperstreet");
	
/*QUAKED trigger_multiple_bcs_mp_det_lowerstreet (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_lowerstreet"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_lowerstreet", "det_lowerstreet");
	
/*QUAKED trigger_multiple_bcs_mp_det_bycontainers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_bycontainers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_bycontainers", "det_bycontainers");
	
/*QUAKED trigger_multiple_bcs_mp_det_bydiner (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="det_bydiner"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_det_bydiner", "det_bydiner");

}

mp_greenband()
{
/*QUAKED trigger_multiple_bcs_mp_grn_2ndflooraquarium  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_2ndflooraquarium"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_2ndflooraquarium", "grn_2ndflooraquarium");
	
/*QUAKED trigger_multiple_bcs_mp_grn_hotelbar  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_hotelbar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_hotelbar", "grn_hotelbar");
	
/*QUAKED trigger_multiple_bcs_mp_grn_incafe  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_incafe"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_incafe", "grn_incafe");

/*QUAKED trigger_multiple_bcs_mp_grn_behindcherrytree  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_behindcherrytree"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_behindcherrytree", "grn_behindcherrytree");
	
/*QUAKED trigger_multiple_bcs_mp_grn_aquariumhallway  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_aquariumhallway"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_aquariumhallway", "grn_aquariumhallway");
	
/*QUAKED trigger_multiple_bcs_mp_grn_aquariumpatio  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_aquariumpatio"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_aquariumpatio",	"grn_aquariumpatio");
	
/*QUAKED trigger_multiple_bcs_mp_grn_inelevator  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_inelevator"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_inelevator", "grn_inelevator");
	
/*QUAKED trigger_multiple_bcs_mp_grn_behindaquariumdesk  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_behindaquariumdesk"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_behindaquariumdesk", "grn_behindaquariumdesk");
	
/*QUAKED trigger_multiple_bcs_mp_grn_inzengarden  (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_inzengarden"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_inzengarden", "grn_inzengarden");
	
/*QUAKED trigger_multiple_bcs_mp_grn_topofzengarden (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_topofzengarden"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_topofzengarden", "grn_topofzengarden");
	
/*QUAKED trigger_multiple_bcs_mp_grn_hotelentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_hotelentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_hotelentrance", "grn_hotelentrance");
	
/*QUAKED trigger_multiple_bcs_mp_grn_hotellobby (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_hotellobby"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_hotellobby", "grn_hotellobby");
	
/*QUAKED trigger_multiple_bcs_mp_grn_inaquarium (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_inaquarium"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_inaquarium", "grn_aquarium");

/*QUAKED trigger_multiple_bcs_mp_grn_inlounge (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_inlounge"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_inlounge", "grn_inlounge");
	
/*QUAKED trigger_multiple_bcs_mp_grn_behindstatue (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_behindstatue"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_behindstatue", "grn_behindstatue");
	
/*QUAKED trigger_multiple_bcs_mp_grn_insidewalkway (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_insidewalkway"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_insidewalkway", "grn_insidewalkway");
	
/*QUAKED trigger_multiple_bcs_mp_grn_underawning (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_underawning"
*/ 

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_underawning", "grn_underawning");
	
/*QUAKED trigger_multiple_bcs_mp_grn_topofawning (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_topofawning" 
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_topofawning", "grn_topofawning");

/*QUAKED trigger_multiple_bcs_mp_grn_behindlargerock (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_behindlargerock" 
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_behindlargerock", "grn_behindlargerock");
	
/*QUAKED trigger_multiple_bcs_mp_grn_nearsculpture (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_nearsculpture"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_nearsculpture", "grn_nearsculpture");
	
/*QUAKED trigger_multiple_bcs_mp_grn_behindsmallplanter (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="grn_behindsmallplanter" 
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_grn_behindsmallplanter", "grn_behindsmallplanter");
	
}

mp_instinct()
{
/*QUAKED trigger_multiple_bcs_mp_ins_byriverbed (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_byriverbed
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_byriverbed", "ins_byriverbed");

/*QUAKED trigger_multiple_bcs_mp_ins_underexcavator (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_underexcavator"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_underexcavator", "ins_underexcavator");
	
/*QUAKED trigger_multiple_bcs_mp_ins_oncliffs (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_oncliffs"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_oncliffs", "ins_oncliffs");
	
/*QUAKED trigger_multiple_bcs_mp_ins_centertemple (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_centertemple"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_centertemple", "ins_centertemple");
	
/*QUAKED trigger_multiple_bcs_mp_ins_towerofruins (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_towerofruins"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_towerofruins", "ins_towerofruins");
	
/*QUAKED trigger_multiple_bcs_mp_ins_nearpryramidhall (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_nearpryramidhall"
*/

	add_bcs_location_mapping( "triger_multiple_bcs_mp_ins_nearpryramidhall", "ins_nearpryramidhall");
	
/*QUAKED trigger_multiple_bcs_mp_ins_insidepryramid (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_insidepryramid"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_insidepryramid", "ins_insidepryramid");
	
/*QUAKED trigger_multiple_bcs_mp_ins_behindtrailers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_behindtrailers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_behindtrailers", "ins_behindtrailers");
	
/*QUAKED trigger_multiple_bcs_mp_ins_powergenerator (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_powergenerators"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_powergenerators", "ins_powergenerators");
	
/*QUAKED trigger_multiple_bcs_mp_ins_onrubble (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="ins_onrubble"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_ins_onrubble", "ins_onrubble");
	
}

mp_levity()
{
/*QUAKED trigger_multiple_bcs_mp_lev_secgates (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_secgates"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_secgates", "lev_secgates");
	
/*QUAKED trigger_multiple_bcs_mp_lev_controlroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_controlroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_controlroom", "lev_controlroom");
	
/*QUAKED trigger_multiple_bcs_mp_lev_accessroofs (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_accessroofs"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_accessroofs", "lev_accessroofs");
	
/*QUAKED trigger_multiple_bcs_mp_lev_mainhangar (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_mainhangar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_mainhangar", "lev_mainhangar");
	
/*QUAKED trigger_multiple_bcs_mp_lev_dronestorage (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_dronestorage"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_dronestorage", "lev_dronestorage");
	
/*QUAKED trigger_multiple_bcs_mp_lev_inflightcontrol (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_inflightcontrol"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_inflightcontrol", "lev_flightcontrol");
	
/*QUAKED trigger_multiple_bcs_mp_lev_flightcontrolroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_flightcontrolroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_flightcontrolroof", "lev_flightcontrolroof");
	
/*QUAKED trigger_multiple_bcs_mp_lev_flightcontrolalley (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_flightcontrolalley"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_flightcontrolalley", "lev_flightcontrolalley");
	
/*QUAKED trigger_multiple_bcs_mp_lev_observationdeck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_observationdeck"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_observationdeck", "lev_observationdeck");
	
/*QUAKED trigger_multiple_bcs_mp_lev_severaccessroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_serveraccessroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_serveraccessroof", "lev_serveraccessroof");
	
/*QUAKED trigger_multiple_bcs_mp_lev_hangarlounge (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_hangarlounge"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_hangarlounge", "lev_hangarlounge");
	
/*QUAKED trigger_multiple_bcs_mp_lev_powercontrol (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_powercontrol"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_powercontrol", "lev_powercontrol");
	
/*QUAKED trigger_multiple_bcs_mp_lev_readyrooms (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_readyrooms"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_readyrooms", "lev_readyrooms");
	
/*QUAKED trigger_multiple_bcs_mp_lev_rearhangar (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_rearhangar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_rearhangar", "lev_rearhangar");
	
/*QUAKED trigger_multiple_bcs_mp_lev_serveraccess (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_serveraccess"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_serveraccess", "lev_serveraccess");
	
/*QUAKED trigger_multiple_bcs_mp_lev_nearwaterfall (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_nearwaterfall"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_nearwaterfall", "lev_nearwaterfall");
	
/*QUAKED trigger_multiple_bcs_mp_lev_nearrockslide (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="lev_nearrockslide"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_lev_nearrockslide", "lev_nearrockslide");
	
}

mp_recovery()
{
/*QUAKED trigger_multiple_bcs_mp_rec_inobservatory (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_inobservatory"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_inobservatory", "rec_inobservatory");
	
/*QUAKED trigger_multiple_bcs_mp_rec_bytram (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_bytram"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_bytram", "rec_bytram");
	
/*QUAKED trigger_multiple_bcs_mp_rec_byskywalk (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_byskywalk"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_byskywalk", "rec_byskywalk");
	
/*QUAKED trigger_multiple_bcs_mp_rec_inravine (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_inravine"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_inravine", "rec_inravine");
	
/*QUAKED trigger_multiple_bcs_mp_rec_insiderockhall (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_insiderockhall"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_insiderockhall", "rec_insiderockhall");
	
/*QUAKED trigger_multiple_bcs_mp_rec_byhangardoor (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_byhangardoor"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_byhangardoor", "rec_byhangardoor");
	
/*QUAKED trigger_multiple_bcs_mp_rec_inplaza (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_inplaza"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_inplaza", "rec_inplaza");
	
/*QUAKED trigger_multiple_bcs_mp_rec_bychopper (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_bychopper"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_bychopper", "rec_bychopper");
	
/*QUAKED trigger_multiple_bcs_mp_rec_onlaunchdeck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_onlaunchdeck"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_onlaunchdeck", "rec_onlaunchdeck");
	
/*QUAKED trigger_multiple_bcs_mp_rec_nearridge (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_nearridge"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_nearridge", "rec_nearridge");
	
/*QUAKED trigger_multiple_bcs_mp_rec_undercontroltower (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_undercontroltower"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_undercontroltower", "rec_undercontroltower");
	
/*QUAKED trigger_multiple_bcs_mp_rec_gatecontrolroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_gatecontrolroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_gatecontrolroom", "rec_gatecontrolroom");
	
/*QUAKED trigger_multiple_bcs_mp_rec_underdeck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_underdeck"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_underdeck", "rec_underdeck");
	
/*QUAKED trigger_multiple_bcs_mp_rec_observationdeck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_observationdeck"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_observationdeck", "rec_observationdeck");
	
/*QUAKED trigger_multiple_bcs_mp_rec_byrustedvan (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="rec_byrustedvan"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_rec_byrustedvan", "rec_byrustedvan");

}

mp_solar()
{
/*QUAKED trigger_multiple_bcs_mp_slr_inparkinglot (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_inparkinglot"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_inparkinglot", "slr_inparkinglot");
	
/*QUAKED trigger_multiple_bcs_mp_slr_bymainentry (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_bymainentry"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_bymainentry", "slr_bymainentry");
	
/*QUAKED trigger_multiple_bcs_mp_slr_atpool (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_atpool"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_atpool", "slr_atpool");
	
/*QUAKED trigger_multiple_bcs_mp_slr_indraintunnel (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_indraintunnel"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_indraintunnel", "slr_indraintunnel");
	
/*QUAKED trigger_multiple_bcs_mp_slr_inoffice (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_inoffice"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_inoffice", "slr_inoffice");
	
/*QUAKED trigger_multiple_bcs_mp_slr_bysmalltanks (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_bysmalltanks"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_bysmalltanks", "slr_bysmalltanks");
	
/*QUAKED trigger_multiple_bcs_mp_slr_intowercontrolroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_intowercontrolroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_intowercontrolroom", "slr_intowercontrolroom");
	
/*QUAKED trigger_multiple_bcs_mp_slr_bytransformers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_bytransformers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_bytransformers", "slr_bytransformers");
	
/*QUAKED trigger_multiple_bcs_mp_slr_condensercontrolroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_condensercontrolroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_condensercontrolroom", "slr_condensercontrolroom");
	
/*QUAKED trigger_multiple_bcs_mp_slr_inbacklot (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_inbacklot"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_inbacklot", "slr_inbacklot");
	
/*QUAKED trigger_multiple_bcs_mp_slr_bycyclonetank (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_bycyclonetank"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_bycyclonetank", "slr_bycyclonetank");
	
/*QUAKED trigger_multiple_bcs_mp_slr_indrainarea (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_indrainarea"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_indrainarea", "slr_indrainarea");
	
/*QUAKED trigger_multiple_bcs_mp_slr_indriveway (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_indriveway"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_indriveway", "slr_indriveway");
	
/*QUAKED trigger_multiple_bcs_mp_slr_visitorcenter (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_visitorcenter"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_visitorcenter", "slr_visitorcenter");
	
/*QUAKED trigger_multiple_bcs_mp_slr_inpumproom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_inpumproom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_inpumproom", "slr_inpumproom");
	
/*QUAKED trigger_multiple_bcs_mp_slr_sciencearea (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_sciencearea"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_sciencearea", "slr_sciencearea");
	
/*QUAKED trigger_multiple_bcs_mp_slr_ingarage (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_ingarage"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_ingarage", "slr_ingarage");
	
/*QUAKED trigger_multiple_bcs_mp_slr_oncatwalks (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_oncatwalks"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_oncatwalks", "slr_oncatwalks");
	
/*QUAKED trigger_multiple_bcs_mp_slr_bycondensers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_bycondensers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_bycondensers", "slr_bycondensers");
	
/*QUAKED trigger_multiple_bcs_mp_slr_inutilityroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_inutilityroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_inutilityroom", "slr_inutilityroom");
	
/*QUAKED trigger_multiple_bcs_mp_slr_onutilityroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="slr_onutilityroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_slr_onutilityroof", "slr_onutilityroof");
	
}

mp_terrace()
{
/*QUAKED trigger_multiple_bcs_mp_trc_hotellobby (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_hotellobby"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_hotellobby", "trc_hotellobby");
	
/*QUAKED trigger_multiple_bcs_mp_trc_atgrotto (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_atgrotto"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_atgrotto", "trc_atgrotto");
	
/*QUAKED trigger_multiple_bcs_mp_trc_insaunatunnel (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_insaunatunnel"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_insaunatunnel", "trc_insaunatunnel");
	
/*QUAKED trigger_multiple_bcs_mp_trc_saunaentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_saunaentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_saunaentrance", "trc_saunaentrance");
	
/*QUAKED trigger_multiple_bcs_mp_trc_inlowerruins (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_inlowerruins"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_inlowerruins", "trc_inlowerruins");	
	
/*QUAKED trigger_multiple_bcs_mp_trc_inupperruins (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_inupperruins"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_inupperruins", "trc_inupperruins");
	
/*QUAKED trigger_multiple_bcs_mp_trc_onupperterrace (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_onupperterrace"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_onupperterrace", "trc_onupperterrace");
	
/*QUAKED trigger_multiple_bcs_mp_trc_incafe (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_incafe"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_incafe", "trc_incafe");
	
/*QUAKED trigger_multiple_bcs_mp_trc_undertower (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_undertower"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_undertower", "trc_undertower");
	
/*QUAKED trigger_multiple_bcs_mp_trc_nightclubentrance (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_nightclubentrance"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_nightclubentrance", "trc_nightclubentrance");
	
/*QUAKED trigger_multiple_bcs_mp_trc_yellowroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_yellowroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_yellowroom", "trc_yellowroom");
	
/*QUAKED trigger_multiple_bcs_mp_trc_redroom (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_redroom"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_redroom", "trc_redroom");
	
/*QUAKED trigger_multiple_bcs_mp_trc_lowerterrace (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_lowerterrace"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_lowerterrace", "trc_lowerterrace");	
	
/*QUAKED trigger_multiple_bcs_mp_trc_atbonfire (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_atbonfire"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_atbonfire", "trc_atbonfire");
	
/*QUAKED trigger_multiple_bcs_mp_trc_poseidonspool (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_poseidonspool"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_poseidonspool", "trc_poseidonspool");
	
/*QUAKED trigger_multiple_bcs_mp_trc_saunaroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_saunaroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_saunaroof", "trc_saunaroof");
	
/*QUAKED trigger_multiple_bcs_mp_trc_inshowers (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_inshowers"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_inshowers", "trc_inshowers");
	
/*QUAKED trigger_multiple_bcs_mp_trc_topofminervamall (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_topofminervamall"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_topofminervamall", "trc_topofminervamall");
	
/*QUAKED trigger_multiple_bcs_mp_trc_sixbellspatio (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trc_sixbellspatio"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trc_sixbellspatio", "trc_sixbellspatio");	
	
}

mp_torqued()
{

/*QUAKED trigger_multiple_bcs_mp_trq_nearbusstop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_nearbusstop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_nearbusstop", "trq_nearbusstop");
	
/*QUAKED trigger_multiple_bcs_mp_trq_incafe (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_incafe"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_incafe", "trq_incafe");
	
/*QUAKED trigger_multiple_bcs_mp_trq_chocolateshop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_chocolateshop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_chocolateshop", "trq_chocolateshop");
	
/*QUAKED trigger_multiple_bcs_mp_trq_baseofclocktower (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_baseofclocktower"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_baseofclocktower", "trq_baseofclocktower");
	
/*QUAKED trigger_multiple_bcs_mp_trq_inclocktower (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_inclocktower"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_inclocktower", "trq_inclocktower");
	
/*QUAKED trigger_multiple_bcs_mp_trq_nearfountain (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_nearfountain"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_nearfountain", "trq_nearfountain");

/*QUAKED trigger_multiple_bcs_mp_trq_outdoorcafe (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_outdoorcafe"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_outdoorcafe", "trq_outdoorcafe");
	
/*QUAKED trigger_multiple_bcs_mp_trq_parkinglot (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_parkinglot"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_parkinglot", "trq_parkinglot");
	
/*QUAKED trigger_multiple_bcs_mp_trq_gatedpatio (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_gatedpatio"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_gatedpatio", "trq_gatedpatio");
	
/*QUAKED trigger_multiple_bcs_mp_trq_nearrestaurant (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_nearrestaurant"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_nearrestaurant", "trq_nearrestaurant");
	
/*QUAKED trigger_multiple_bcs_mp_trq_insemi (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_insemi"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_insemi", "trq_insemi");
	
/*QUAKED trigger_multiple_bcs_mp_trq_nearsnackbar (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_nearsnackbar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_nearsnackbar", "trq_nearsnackbar");
	
/*QUAKED trigger_multiple_bcs_mp_trq_insideticketcounter (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_insideticketcounter"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_insideticketcounter", "trq_insideticketcounter");
	
/*QUAKED trigger_multiple_bcs_mp_trq_frontoftrain (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_frontoftrain"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_frontoftrain", "trq_frontoftrain");
	
/*QUAKED trigger_multiple_bcs_mp_trq_backoftrain (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_backoftrain"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_backoftrain", "trq_backoftrain");
	
/*QUAKED trigger_multiple_bcs_mp_trq_utilitywalkway (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_utilitywalkway"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_utilitywalkway", "trq_utilitywalkway");
	
/*QUAKED trigger_multiple_bcs_mp_trq_onskywalk (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_onskywalk"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_onskywalk", "trq_onskywalk");
	
/*QUAKED trigger_multiple_bcs_mp_trq_topofawning (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_topofawning"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_topofawning", "trq_topofawning");
	
/*QUAKED trigger_multiple_bcs_mp_trq_behindbearstatue (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_behindbearstatue"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_behindbearstatue", "trq_behindbearstatue");
	
/*QUAKED trigger_multiple_bcs_mp_trq_behindcar (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_behindcar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_behindcar", "trq_behindcar");
	
/*QUAKED trigger_multiple_bcs_mp_trq_incoffeeshop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_incoffeeshop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_incoffeeshop", "trq_incoffeeshop");
	
/*QUAKED trigger_multiple_bcs_mp_trq_inconstructionblgd (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_inconstructionblgd"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_inconstructionblgd", "trq_inconstructionblgd");
	
/*QUAKED trigger_multiple_bcs_mp_trq_inelevator (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_inelevator"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_inelevator", "trq_inelevator");
	
/*QUAKED trigger_multiple_bcs_mp_trq_onroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_onroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_onroof", "trq_onroof");
	
/*QUAKED trigger_multiple_bcs_mp_trq_behindsecuritytruck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_behindsecuritytruck"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_behindsecuritytruck", "trq_behindsecuritytruck");
	
/*QUAKED trigger_multiple_bcs_mp_trq_aboveticketcounter (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="trq_aboveticketcounter"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_trq_aboveticketcounter", "trq_aboveticketcounter");
}

mp_venus()
{
/*QUAKED trigger_multiple_bcs_mp_vns_inpool (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_inpool"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_inpool", "vns_inpool");
	
/*QUAKED trigger_multiple_bcs_mp_vns_bypool (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_bypool"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_bypool", "vns_bypool");
	
/*QUAKED trigger_multiple_bcs_mp_vns_inlounge (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_inlounge"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_inlounge", "vns_inlounge");
	
/*QUAKED trigger_multiple_bcs_mp_vns_inlobby (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_inlobby"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_inlobby", "vns_inlobby");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onbalcony (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onbalcony"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onbalcony", "vns_onbalcony");
	
/*QUAKED trigger_multiple_bcs_mp_vns_byhelipad (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_byhelipad"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_byhelipad", "vns_byhelipad");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onfountain (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onfountain"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onfountain", "vns_onfountain");
	
/*QUAKED trigger_multiple_bcs_mp_vns_byfountain (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_byfountain"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_byfountain", "vns_byfountain");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onpoolshade (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onpoolshade"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onpoolshade", "vns_onpoolshade");
	
/*QUAKED trigger_multiple_bcs_mp_vns_inzengarden (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_inzengarden"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_inzengarden", "vns_inzengarden");
	
/*QUAKED trigger_multiple_bcs_mp_vns_inbar (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_inbar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_inbar", "vns_inbar");
	
/*QUAKED trigger_multiple_bcs_mp_vns_bybar (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_bybar"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_bybar", "vns_bybar");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onwoodendeck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onwoodendeck"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onwoodendeck", "vns_onwoodendeck");
	
/*QUAKED trigger_multiple_bcs_mp_vns_inobservationlounge (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_inobservationlounge"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_inobservationlounge", "vns_inobservationlounge");
	
/*QUAKED trigger_multiple_bcs_mp_vns_bywaterfall (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_bywaterfall"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_bywaterfall", "vns_bywaterfall");
	
/*QUAKED trigger_multiple_bcs_mp_vns_incentercourtyard (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_incentercourtyard"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_incentercourtyard", "vns_incentercourtyard");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onpatio (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onpatio"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onpatio", "vns_onpatio");
	
/*QUAKED trigger_multiple_bcs_mp_vns_byfallentree (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_byfallentree"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_byfallentree", "vns_byfallentree");
	
/*QUAKED trigger_multiple_bcs_mp_vns_inbansaigarden (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_inbansaigarden"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_inbansaigarden", "vns_inbansaigarden");
	
/*QUAKED trigger_multiple_bcs_mp_vns_bypenthouse (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_bypenthouse"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_bypenthouse", "vns_bypenthouse");
	
/*QUAKED trigger_multiple_bcs_mp_vns_byhorsestatue (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_byhorsestatue"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_byhorsestatue", "vns_byhorsestatue");
	
/*QUAKED trigger_multiple_bcs_mp_vns_bysoliderstatue (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_bysoliderstatue"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_bysoliderstatue", "vns_bysoliderstatue");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onhallwayrooftop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onhallwayrooftop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onhallwayrooftop", "vns_onhallwayrooftop");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onbalconystaircase (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onbalconystaircase"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onbalconystaircase", "vns_onbalconystaircase");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onoutsidebalconystaircase (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onoutsidebalconystaircase"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onoutsidebalconystaircase", "vns_onoutsidebalconystaircase");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onobservationdeck (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onobservationdeck"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onobservationdeck", "vns_onobservationdeck");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onobservationrooftop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onobservationrooftop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onobservationrooftop", "vns_onobservationrooftop");
	
/*QUAKED trigger_multiple_bcs_mp_vns_underwaterfall (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_underwaterfall"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_underwaterfall", "vns_underwaterfall");
	
/*QUAKED trigger_multiple_bcs_mp_vns_insaunalobby (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_insaunalobby"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_insaunalobby", "vns_insaunalobby");
	
/*QUAKED trigger_multiple_bcs_mp_vns_underkeyholearchway (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_underkeyholearchway"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_underkeyholearchway", "vns_underkeyholearchway");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onoverhang (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onoverhang"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onoverhang", "vns_onoverhang");
	
/*QUAKED trigger_multiple_bcs_mp_vns_bylanterns (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_bylanterns"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_bylanterns", "vns_bylanterns");
	
/*QUAKED trigger_multiple_bcs_mp_vns_onpenthouseroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="vns_onpenthouseroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_vns_onpenthouseroof", "vns_onpenthouseroof");
}

mp_comeback()
{
/*QUAKED trigger_multiple_bcs_mp_cbk_rearentrancenetcafe (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_rearentrancenetcafe"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_rearentrancenetcafe", "cbk_rearentrancenetcafe");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_netcafe (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_netcafe"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_netcafe", "cbk_netcafe");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_videostore (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_videostore"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_videostore", "cbk_videostore");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_videostoreroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_videostoreroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_videostoreroof", "cbk_videostoreroof");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_byshanty (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_byshanty"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_byshanty", "cbk_byshanty");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_powerstationroof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_powerstationroof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_powerstationroof", "cbk_powerstationroof");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_centercoil (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_centercoil"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_centercoil", "cbk_centercoil");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_glassoffices (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_glassoffices"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_glassoffices", "cbk_glassoffices");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_grassybalcony (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_grassybalcony"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_grassybalcony", "cbk_grassybalcony");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_fabricsshop (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_fabricsshop"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_fabricsshop", "cbk_fabricsshop");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_fabricsshoproof (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_fabricsshoproof"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_fabricsshoproof", "cbk_fabricsshoproof");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_westcourtyard (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_westcourtyard"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_westcourtyard", "cbk_westcourtyard");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_nearskywalk (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_nearskywalk"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_nearskywalk", "cbk_nearskywalk");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_baseofapttower (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_baseofapttower"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_baseofapttower", "cbk_baseofapttower");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_northcourtyard (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_northcourtyard"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_northcourtyard", "cbk_northcourtyard");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_atopensewer (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_atopensewer"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_atopensewer", "cbk_atopensewer");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_eastcourtyard (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_eastcourtyard"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_eastcourtyard", "cbk_eastcourtyard");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_nearmarket (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_nearmarket"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_nearmarket", "cbk_nearmarket");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_oneonezero (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_oneonezero"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_oneonezero", "cbk_oneonezero");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_southcourtyard (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_southcourtyard"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_southcourtyard", "cbk_southcourtyard");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_southerncourtyard (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_southerncourtyard"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_southerncourtyard", "cbk_southerncourtyard");
	
/*QUAKED trigger_multiple_bcs_mp_cbk_outsideofficebldg (0 0.25 0.5)?
defaulttexture="bcs"
soundalias="cbk_outsideofficebldg"
*/

	add_bcs_location_mapping( "trigger_multiple_bcs_mp_cbk_outsideofficebldg", "cbk_outsideofficebldg");
}

//---------------------------------------------------------
// DELETE THESE
//---------------------------------------------------------
old_locations()
{
/*QUAKED trigger_multiple_bcs_ns_acrosschasm (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="acrosschasm"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_acrosschasm", "acrosschasm" );

/*QUAKED trigger_multiple_bcs_ns_amcrt_stck (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="amcrt_stck"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_amcrt_stck", "amcrt_stck" );

/*QUAKED trigger_multiple_bcs_ns_barr_conc (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="barr_conc"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_barr_conc", "barr_conc" );

/*QUAKED trigger_multiple_bcs_ns_brls (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="brls"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_brls", "brls" );

/*QUAKED trigger_multiple_bcs_ns_catwlk (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="catwlk"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_catwlk", "catwlk" );

/*QUAKED trigger_multiple_bcs_ns_cell_l (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cell_l"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cell_l", "cell_l" );

/*QUAKED trigger_multiple_bcs_ns_cell_r (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cell_r"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cell_r", "cell_r" );

/*QUAKED trigger_multiple_bcs_ns_celldr_endhl (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="celldr_endhl"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_celldr_endhl", "celldr_endhl" );

/*QUAKED trigger_multiple_bcs_ns_corrgatedmtl (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="corrgatedmtl"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_corrgatedmtl", "corrgatedmtl" );

/*QUAKED trigger_multiple_bcs_ns_cot (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cot"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cot", "cot" );

/*QUAKED trigger_multiple_bcs_ns_crt_stck (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="crt_stck"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_crt_stck", "crt_stck" );

/*QUAKED trigger_multiple_bcs_ns_crtstk_nrldge (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="crtstk_nrldge"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_crtstk_nrldge", "crtstk_nrldge" );

/*QUAKED trigger_multiple_bcs_ns_cv_cent (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cv_cent"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cv_cent", "cv_cent" );

/*QUAKED trigger_multiple_bcs_ns_cv_cent_concsup (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cv_cent_concsup"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cv_cent_concsup", "cv_cent_concsup" );

/*QUAKED trigger_multiple_bcs_ns_cv_cent_tv (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cv_cent_tv"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cv_cent_tv", "cv_cent_tv" );

/*QUAKED trigger_multiple_bcs_ns_cv_small_l (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cv_small_l"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cv_small_l", "cv_small_l" );

/*QUAKED trigger_multiple_bcs_ns_cv_wall_inside (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cv_wall_inside"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cv_wall_inside", "cv_wall_inside" );

/*QUAKED trigger_multiple_bcs_ns_cv_wall_outside (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cv_wall_outside"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_cv_wall_outside", "cv_wall_outside" );

/*QUAKED trigger_multiple_bcs_ns_dpstr (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="dpstr"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_dpstr", "dpstr" );

/*QUAKED trigger_multiple_bcs_ns_drvwy (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="drvwy"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_drvwy", "drvwy" );

/*QUAKED trigger_multiple_bcs_ns_dsk_lg (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="dsk_lg"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_dsk_lg", "dsk_lg" );

/*QUAKED trigger_multiple_bcs_ns_dsk_stck (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="dsk_stck"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_dsk_stck", "dsk_stck" );

/*QUAKED trigger_multiple_bcs_ns_fuelcont (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fuelcont"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_fuelcont", "fuelcont" );

/*QUAKED trigger_multiple_bcs_ns_fuelconts (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fuelconts"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_fuelconts", "fuelconts" );

/*QUAKED trigger_multiple_bcs_ns_gbgcns (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="gbgcns"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_gbgcns", "gbgcns" );

/*QUAKED trigger_multiple_bcs_ns_hdghog (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="hdghog"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_hdghog", "hdghog" );

/*QUAKED trigger_multiple_bcs_ns_hesco_nrledge (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="hesco_nrledge"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_hesco_nrledge", "hesco_nrledge" );

/*QUAKED trigger_multiple_bcs_ns_hescobarr (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="hescobarr"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_hescobarr", "hescobarr" );

/*QUAKED trigger_multiple_bcs_ns_icemach (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="icemach"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_icemach", "icemach" );

/*QUAKED trigger_multiple_bcs_ns_intsec_3w (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="intsec_3w"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_intsec_3w", "intsec_3w" );

/*QUAKED trigger_multiple_bcs_ns_lckr_cntr (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lckr_cntr"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_lckr_cntr", "lckr_cntr" );

/*QUAKED trigger_multiple_bcs_ns_lckr_l (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lckr_l"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_lckr_l", "lckr_l" );

/*QUAKED trigger_multiple_bcs_ns_lckr_ne (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lckr_ne"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_lckr_ne", "lckr_ne" );

/*QUAKED trigger_multiple_bcs_ns_lckr_r (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lckr_r"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_lckr_r", "lckr_r" );

/*QUAKED trigger_multiple_bcs_ns_lckr_sw (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lckr_sw"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_lckr_sw", "lckr_sw" );

/*QUAKED trigger_multiple_bcs_ns_lowwall_bwire (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="lowwall_bwire"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_lowwall_bwire", "lowwall_bwire" );

/*QUAKED trigger_multiple_bcs_ns_newsbox (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="newsbox"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_newsbox", "newsbox" );

/*QUAKED trigger_multiple_bcs_ns_phnbth (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="phnbth"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_phnbth", "phnbth" );

/*QUAKED trigger_multiple_bcs_ns_pipes_behind (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="pipes_behind"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_pipes_behind", "pipes_behind" );

/*QUAKED trigger_multiple_bcs_ns_pipes_nside (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="pipes_nside"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_pipes_nside", "pipes_nside" );

/*QUAKED trigger_multiple_bcs_ns_rappel_left (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rappel_left"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_rappel_left", "rappel_left" );

/*QUAKED trigger_multiple_bcs_ns_samlnchr (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="samlnchr"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_samlnchr", "samlnchr" );

/*QUAKED trigger_multiple_bcs_ns_sentrygun (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sentrygun"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_sentrygun", "sentrygun" );

/*QUAKED trigger_multiple_bcs_ns_shwr_cntr (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="shwr_cntr"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_shwr_cntr", "shwr_cntr" );

/*QUAKED trigger_multiple_bcs_ns_shwr_ne (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="shwr_ne"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_shwr_ne", "shwr_ne" );

/*QUAKED trigger_multiple_bcs_ns_shwr_sw (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="shwr_sw"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_shwr_sw", "shwr_sw" );

/*QUAKED trigger_multiple_bcs_ns_sndbgs (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="sndbgs"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_sndbgs", "sndbgs" );

/*QUAKED trigger_multiple_bcs_ns_stairs_down (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="stairs_down"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_stairs_down", "stairs_down" );

/*QUAKED trigger_multiple_bcs_ns_stairs_up (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="stairs_up"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_stairs_up", "stairs_up" );

/*QUAKED trigger_multiple_bcs_ns_stairs_ylw (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="stairs_ylw"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_stairs_ylw", "stairs_ylw" );

/*QUAKED trigger_multiple_bcs_ns_tun_leadoutside (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="tun_leadoutside"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_tun_leadoutside", "tun_leadoutside" );

/*QUAKED trigger_multiple_bcs_ns_vendmach (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="vendmach"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_vendmach", "vendmach" );

/*QUAKED trigger_multiple_bcs_ns_wirespl_lg (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="wirespl_lg"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_wirespl_lg", "wirespl_lg" );

/*QUAKED trigger_multiple_bcs_ns_wlkwy_abv_archs (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="wlkwy_abv_archs"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_ns_wlkwy_abv_archs", "wlkwy_abv_archs" );
	
/*QUAKED trigger_multiple_bcs_df_monument_courtyard (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="monument_courtyard"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_monument_courtyard", "monument_courtyard" );

/*QUAKED trigger_multiple_bcs_df_monument_top (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="monument_top"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_monument_top", "monument_top" );

/*QUAKED trigger_multiple_bcs_df_car_parked (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="car_parked"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_car_parked", "car_parked" );

/*QUAKED trigger_multiple_bcs_df_embassy (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="embassy"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_embassy", "embassy" );

/*QUAKED trigger_multiple_bcs_df_embassy_1st (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="embassy_1st"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_embassy_1st", "embassy_1st" );

/*QUAKED trigger_multiple_bcs_df_embassy_3rd (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="embassy_3rd"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_embassy_3rd", "embassy_3rd" );

/*QUAKED trigger_multiple_bcs_df_vehicle_snowcat (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="vehicle_snowcat"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_vehicle_snowcat", "vehicle_snowcat" );

/*QUAKED trigger_multiple_bcs_df_vehicle_dumptruck (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="vehicle_dumptruck"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_vehicle_dumptruck", "vehicle_dumptruck" );

/*QUAKED trigger_multiple_bcs_df_building_red (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="building_red"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_building_red", "building_red" );

/*QUAKED trigger_multiple_bcs_df_vehicle_snowmobile (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="vehicle_snowmobile"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_vehicle_snowmobile", "vehicle_snowmobile" );

/*QUAKED trigger_multiple_bcs_df_scaffolding_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="scaffolding_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_scaffolding_generic", "scaffolding_generic" );

/*QUAKED trigger_multiple_bcs_df_container_red (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="container_red"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_container_red", "container_red" );

/*QUAKED trigger_multiple_bcs_df_tires_large (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="tires_large"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_tires_large", "tires_large" );

/*QUAKED trigger_multiple_bcs_df_memorial_building (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="memorial_building"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_memorial_building", "memorial_building" );

/*QUAKED trigger_multiple_bcs_df_stand_hotdog (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="stand_hotdog"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_stand_hotdog", "stand_hotdog" );

/*QUAKED trigger_multiple_bcs_df_stand_trading (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="stand_trading"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_stand_trading", "stand_trading" );

/*QUAKED trigger_multiple_bcs_df_subway_entrance (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="subway_entrance"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_subway_entrance", "subway_entrance" );

/*QUAKED trigger_multiple_bcs_df_rubble_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="rubble_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_rubble_generic", "rubble_generic" );

/*QUAKED trigger_multiple_bcs_df_cases_right (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cases_right"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_cases_right", "cases_right" );

/*QUAKED trigger_multiple_bcs_df_cases_left (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cases_left"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_cases_left", "cases_left" );

/*QUAKED trigger_multiple_bcs_df_cases_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="cases_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_cases_generic", "cases_generic" );

/*QUAKED trigger_multiple_bcs_df_barrier_orange (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="barrier_orange"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_barrier_orange", "barrier_orange" );

/*QUAKED trigger_multiple_bcs_df_barrier_hesco (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="barrier_hesco"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_barrier_hesco", "barrier_hesco" );

/*QUAKED trigger_multiple_bcs_df_stryker_destroyed (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="stryker_destroyed"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_stryker_destroyed", "stryker_destroyed" );

/*QUAKED trigger_multiple_bcs_df_fan_exhaust (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="fan_exhaust"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_fan_exhaust", "fan_exhaust" );

/*QUAKED trigger_multiple_bcs_df_tower_jamming (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="tower_jamming"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_tower_jamming", "tower_jamming" );

/*QUAKED trigger_multiple_bcs_df_ac_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="ac_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_ac_generic", "ac_generic" );

/*QUAKED trigger_multiple_bcs_df_table_computer (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="table_computer"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_table_computer", "table_computer" );

/*QUAKED trigger_multiple_bcs_df_bulkhead_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bulkhead_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_bulkhead_generic", "bulkhead_generic" );

/*QUAKED trigger_multiple_bcs_df_bunk_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bunk_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_bunk_generic", "bunk_generic" );

/*QUAKED trigger_multiple_bcs_df_console_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="console_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_console_generic", "console_generic" );

/*QUAKED trigger_multiple_bcs_df_deck_generic (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="deck_generic"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_df_deck_generic", "deck_generic" );


}

old_locations_mp()
{
/*QUAKED trigger_multiple_bcs_mp_dome_bunker (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bunker"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_bunker", "bunker" );

/*QUAKED trigger_multiple_bcs_mp_dome_bunker_back (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="bunker_back"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_bunker_back", "bunker_back" );

/*QUAKED trigger_multiple_bcs_mp_dome_office (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="office"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_office", "office" );

/*QUAKED trigger_multiple_bcs_mp_dome_dome (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="dome"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_dome", "dome" );

/*QUAKED trigger_multiple_bcs_mp_dome_catwalk (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="catwalk"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_catwalk", "catwalk" );

/*QUAKED trigger_multiple_bcs_mp_dome_loadingbay (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="loadingbay"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_loadingbay", "loadingbay" );

/*QUAKED trigger_multiple_bcs_mp_dome_hallway (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="hallway"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_hallway", "hallway" );

/*QUAKED trigger_multiple_bcs_mp_dome_hallway_loadingbay (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="hallway_loadingbay"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_hallway_loadingbay", "hallway_loadingbay" );

/*QUAKED trigger_multiple_bcs_mp_dome_hallway_office (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="hallway_office"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_hallway_office", "hallway_office" );

/*QUAKED trigger_multiple_bcs_mp_dome_wall_broken (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="wall_broken"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_wall_broken", "wall_broken" );

/*QUAKED trigger_multiple_bcs_mp_dome_tank (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="tank"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_tank", "tank" );

/*QUAKED trigger_multiple_bcs_mp_dome_radar (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="radar"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_radar", "radar" );

/*QUAKED trigger_multiple_bcs_mp_dome_humvee (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="humvee"
*/
	add_bcs_location_mapping( "trigger_multiple_bcs_mp_dome_humvee", "humvee" );
}

/*EXAMPLEQUAKED trigger_multiple_bcs_df_parisAC130_lm_embassy (0 0.25 0.5) ?
defaulttexture="bcs"
soundalias="DF_1_lm_embassy"
*/
//	add_bcs_location_mapping( "trigger_multiple_bcs_df_parisAC130_lm_embassy", "lm_embassy" );
